@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2016 Brazil
+  Copyright(C) 2009-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -75,6 +75,8 @@ typedef struct {
   grn_obj level;
   grn_obj columns;
   grn_obj ids;
+  grn_obj return_codes;
+  grn_obj error_messages;
   uint32_t emit_level;
   int32_t id_offset;  /* Position of _id in values or -1 if _id is N/A. */
   int32_t key_offset; /* Position of _key in values or -1 if _key is N/A. */
@@ -85,12 +87,15 @@ typedef struct {
   uint32_t unichar;
   uint32_t values_size;
   uint32_t nrecords;
+  uint32_t n_record_errors;
+  uint32_t n_column_errors;
   grn_loader_stat stat;
   grn_content_type input_type;
   grn_loader_columns_status columns_status;
   grn_rc rc;
   char errbuf[GRN_CTX_MSGSIZE];
   grn_bool output_ids;
+  grn_bool output_errors;
 } grn_loader;
 
 #define GRN_CTX_N_SEGMENTS 512
@@ -158,8 +163,9 @@ struct _grn_ctx_impl {
 #endif
 
   /* expression portion */
-  grn_obj *stack[GRN_STACK_SIZE];
+  grn_obj **stack;
   uint32_t stack_curr;
+  uint32_t stack_size;
   grn_hash *expr_vars;
   grn_obj *curr_expr;
   grn_obj current_request_id;
