@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013-2016 Brazil
+  Copyright(C) 2013-2017 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,7 @@
 #include <mruby/hash.h>
 
 #include "../grn_expr.h"
+#include "../grn_proc.h"
 #include "../grn_util.h"
 #include "../grn_mrb.h"
 #include "mrb_accessor.h"
@@ -104,11 +105,11 @@ mrb_grn_scan_info_put_index(mrb_state *mrb, mrb_value self)
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   scan_info *si;
   mrb_value mrb_index;
-  int sid;
-  int32_t weight;
+  mrb_int sid;
+  mrb_int weight;
   mrb_value mrb_scorer;
   mrb_value mrb_scorer_args_expr;
-  int32_t scorer_args_expr_offset;
+  mrb_int scorer_args_expr_offset;
   grn_obj *index;
   grn_obj *scorer = NULL;
   grn_obj *scorer_args_expr = NULL;
@@ -162,7 +163,7 @@ static mrb_value
 mrb_grn_scan_info_set_end(mrb_state *mrb, mrb_value self)
 {
   scan_info *si;
-  int end;
+  mrb_int end;
 
   mrb_get_args(mrb, "i", &end);
   si = DATA_PTR(self);
@@ -190,7 +191,7 @@ static mrb_value
 mrb_grn_scan_info_set_flags(mrb_state *mrb, mrb_value self)
 {
   scan_info *si;
-  int flags;
+  mrb_int flags;
 
   mrb_get_args(mrb, "i", &flags);
   si = DATA_PTR(self);
@@ -238,7 +239,7 @@ static mrb_value
 mrb_grn_scan_info_set_max_interval(mrb_state *mrb, mrb_value self)
 {
   scan_info *si;
-  int max_interval;
+  mrb_int max_interval;
 
   mrb_get_args(mrb, "i", &max_interval);
   si = DATA_PTR(self);
@@ -261,7 +262,7 @@ static mrb_value
 mrb_grn_scan_info_set_similarity_threshold(mrb_state *mrb, mrb_value self)
 {
   scan_info *si;
-  int similarity_threshold;
+  mrb_int similarity_threshold;
 
   mrb_get_args(mrb, "i", &similarity_threshold);
   si = DATA_PTR(self);
@@ -285,7 +286,7 @@ mrb_grn_scan_info_get_arg(mrb_state *mrb, mrb_value self)
 {
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   scan_info *si;
-  int index;
+  mrb_int index;
   grn_obj *arg;
 
   mrb_get_args(mrb, "i", &index);
@@ -299,6 +300,7 @@ mrb_grn_scan_info_get_arg(mrb_state *mrb, mrb_value self)
 static mrb_value
 mrb_grn_scan_info_push_arg(mrb_state *mrb, mrb_value self)
 {
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
   scan_info *si;
   mrb_value mrb_arg;
   grn_bool success;
@@ -306,7 +308,7 @@ mrb_grn_scan_info_push_arg(mrb_state *mrb, mrb_value self)
   mrb_get_args(mrb, "o", &mrb_arg);
 
   si = DATA_PTR(self);
-  success = grn_scan_info_push_arg(si, DATA_PTR(mrb_arg));
+  success = grn_scan_info_push_arg(ctx, si, DATA_PTR(mrb_arg));
 
   return mrb_bool_value(success);
 }
@@ -326,7 +328,7 @@ static mrb_value
 mrb_grn_scan_info_set_start_position(mrb_state *mrb, mrb_value self)
 {
   scan_info *si;
-  int start_position;
+  mrb_int start_position;
 
   mrb_get_args(mrb, "i", &start_position);
   si = DATA_PTR(self);
@@ -380,6 +382,13 @@ mrb_grn_expr_code_inspect(mrb_state *mrb, mrb_value self)
                                0));
   }
 
+  mrb_str_cat_lit(mrb, inspected, ", n_args=");
+  mrb_str_concat(mrb, inspected,
+                 mrb_funcall(mrb,
+                             mrb_fixnum_value(code->nargs),
+                             "inspect",
+                             0));
+
   mrb_str_cat_lit(mrb, inspected, ", modify=");
   mrb_str_concat(mrb, inspected,
                  mrb_funcall(mrb,
@@ -436,6 +445,15 @@ mrb_grn_expr_code_get_value(mrb_state *mrb, mrb_value self)
   return grn_mrb_value_from_grn_obj(mrb, expr_code->value);
 }
 
+static mrb_value
+mrb_grn_expr_code_get_n_args(mrb_state *mrb, mrb_value self)
+{
+  grn_expr_code *expr_code;
+
+  expr_code = DATA_PTR(self);
+  return mrb_fixnum_value(expr_code->nargs);
+}
+
 static mrb_value
 mrb_grn_expr_code_get_op(mrb_state *mrb, mrb_value self)
 {
@@ -454,6 +472,15 @@ mrb_grn_expr_code_get_flags(mrb_state *mrb, mrb_value self)
   return mrb_fixnum_value(expr_code->flags);
 }
 
+static mrb_value
+mrb_grn_expr_code_get_modify(mrb_state *mrb, mrb_value self)
+{
+  grn_expr_code *expr_code;
+
+  expr_code = DATA_PTR(self);
+  return mrb_fixnum_value(expr_code->modify);
+}
+
 static mrb_value
 mrb_grn_expression_class_create(mrb_state *mrb, mrb_value klass)
 {
@@ -500,6 +527,15 @@ mrb_grn_expression_initialize(mrb_state *mrb, mrb_value self)
   return self;
 }
 
+static mrb_value
+mrb_grn_expression_is_empty(mrb_state *mrb, mrb_value self)
+{
+  grn_expr *expr;
+
+  expr = DATA_PTR(self);
+  return mrb_bool_value(expr->codes_curr == 0);
+}
+
 static mrb_value
 mrb_grn_expression_codes(mrb_state *mrb, mrb_value self)
 {
@@ -555,6 +591,28 @@ mrb_grn_expression_array_reference(mrb_state *mrb, mrb_value self)
   return grn_mrb_value_from_grn_obj(mrb, var);
 }
 
+static mrb_value
+mrb_grn_expression_set_condition(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj *expr;
+  mrb_value mrb_condition;
+  grn_obj *condition_ptr;
+
+  mrb_get_args(mrb, "o", &mrb_condition);
+
+  expr = DATA_PTR(self);
+  condition_ptr = grn_expr_get_or_add_var(ctx,
+                                          expr,
+                                          GRN_SELECT_INTERNAL_VAR_CONDITION,
+                                          GRN_SELECT_INTERNAL_VAR_CONDITION_LEN);
+  GRN_OBJ_FIN(ctx, condition_ptr);
+  GRN_PTR_INIT(condition_ptr, 0, GRN_DB_OBJECT);
+  GRN_PTR_SET(ctx, condition_ptr, GRN_MRB_DATA_PTR(mrb_condition));
+
+  return mrb_nil_value();
+}
+
 static mrb_value
 mrb_grn_expression_take_object(mrb_state *mrb, mrb_value self)
 {
@@ -592,6 +650,14 @@ mrb_grn_expression_allocate_constant(mrb_state *mrb, mrb_value self)
     GRN_TEXT_SET(ctx, grn_object,
                  RSTRING_PTR(mrb_object), RSTRING_LEN(mrb_object));
     break;
+  case MRB_TT_TRUE:
+    grn_object = grn_expr_alloc_const(ctx, expr);
+    if (!grn_object) {
+      grn_mrb_ctx_check(mrb);
+    }
+    GRN_BOOL_INIT(grn_object, 0);
+    GRN_BOOL_SET(ctx, grn_object, GRN_TRUE);
+    break;
   default:
     mrb_raisef(mrb, E_ARGUMENT_ERROR, "unsupported type: %S", mrb_object);
     break;
@@ -617,8 +683,13 @@ mrb_grn_expression_parse(mrb_state *mrb, mrb_value self)
   mrb_get_args(mrb, "s|H", &query, &query_size, &mrb_options);
 
   if (!mrb_nil_p(mrb_options)) {
+    mrb_value mrb_default_column;
     mrb_value mrb_flags;
 
+    mrb_default_column =
+      grn_mrb_options_get_lit(mrb, mrb_options, "default_column");
+    default_column = GRN_MRB_DATA_PTR(mrb_default_column);
+
     mrb_flags = grn_mrb_options_get_lit(mrb, mrb_options, "flags");
     if (!mrb_nil_p(mrb_flags)) {
       flags = mrb_fixnum(mrb_flags);
@@ -641,7 +712,7 @@ mrb_grn_expression_append_object(mrb_state *mrb, mrb_value self)
   grn_obj *object;
   mrb_value mrb_op;
   grn_operator op;
-  int n_args;
+  mrb_int n_args;
 
   expr = DATA_PTR(self);
   mrb_get_args(mrb, "ooi", &mrb_object, &mrb_op, &n_args);
@@ -662,7 +733,7 @@ mrb_grn_expression_append_constant(mrb_state *mrb, mrb_value self)
   mrb_value mrb_constant;
   mrb_value mrb_op;
   grn_operator op;
-  int n_args;
+  mrb_int n_args;
 
   expr = DATA_PTR(self);
   mrb_get_args(mrb, "ooi", &mrb_constant, &mrb_op, &n_args);
@@ -756,7 +827,7 @@ mrb_grn_expression_append_operator(mrb_state *mrb, mrb_value self)
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   grn_obj *expr;
   mrb_value mrb_op;
-  int n_args;
+  mrb_int n_args;
   grn_operator op;
 
   expr = DATA_PTR(self);
@@ -829,10 +900,14 @@ grn_mrb_expr_init(grn_ctx *ctx)
                     mrb_grn_expr_code_get_weight, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "value",
                     mrb_grn_expr_code_get_value, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "n_args",
+                    mrb_grn_expr_code_get_n_args, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "op",
                     mrb_grn_expr_code_get_op, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "flags",
                     mrb_grn_expr_code_get_flags, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "modify",
+                    mrb_grn_expr_code_get_modify, MRB_ARGS_NONE());
 
   {
     struct RClass *expression_code_class = klass;
@@ -845,16 +920,35 @@ grn_mrb_expr_init(grn_ctx *ctx)
   klass = mrb_define_class_under(mrb, module, "Expression", object_class);
   MRB_SET_INSTANCE_TT(klass, MRB_TT_DATA);
 
+#define DEFINE_FLAG(name)                               \
+  mrb_define_const(mrb, klass,                          \
+                   #name,                               \
+                   mrb_fixnum_value(GRN_EXPR_ ## name))
+
+  DEFINE_FLAG(SYNTAX_QUERY);
+  DEFINE_FLAG(SYNTAX_SCRIPT);
+  DEFINE_FLAG(SYNTAX_OUTPUT_COLUMNS);
+  DEFINE_FLAG(ALLOW_PRAGMA);
+  DEFINE_FLAG(ALLOW_COLUMN);
+  DEFINE_FLAG(ALLOW_UPDATE);
+  DEFINE_FLAG(ALLOW_LEADING_NOT);
+
+#undef DEFINE_FLAG
+
   mrb_define_class_method(mrb, klass, "create",
                           mrb_grn_expression_class_create,
                           MRB_ARGS_REQ(1));
 
   mrb_define_method(mrb, klass, "initialize",
                     mrb_grn_expression_initialize, MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "empty?",
+                    mrb_grn_expression_is_empty, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "codes",
                     mrb_grn_expression_codes, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "[]",
                     mrb_grn_expression_array_reference, MRB_ARGS_REQ(1));
+  mrb_define_method(mrb, klass, "condition=",
+                    mrb_grn_expression_set_condition, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "take_object",
                     mrb_grn_expression_take_object, MRB_ARGS_REQ(1));
   mrb_define_method(mrb, klass, "allocate_constant",
